# -----------------------------
# Le script calcule le nombre moyen de token en input et output grace a différent tokenizers qu'une LLM
# aura besoin pour classifier les articles par jour, semaine, mois. Il stoque le résultat dans un fichier json (./data/token_count.json)
#
# Ce script ne prend pas de paramètres.
#
# Auteur : Pavlovich Ivan
# -----------------------------

from transformers import AutoTokenizer
import json
import sys
import os
import statistics
from datetime import datetime, timedelta

sys.path.append(os.path.abspath(os.path.join(os.path.dirname(__file__), "../../")))

from testModel.utils import get_article_data
from variables.pubmed import NCDS, NCDS_MESH_TERM, KEYWORDS_MESH_TERM, KEYWORDS_MESH_SUBHEADING, KEYWORDS_MESH_SITE_PROPOSITION, KEYWORDS_MESH_PROPOSITION

def lower_keywords(mesh_terms):
    res = []

    for _, mesh_term in mesh_terms.items():
        if isinstance(mesh_term, list):
            res.append([part.lower() for part in mesh_term])
        else:
            res.append(mesh_term.lower())
    
    return res

def mesh_term_present(article_mesh_terms, mesh_term):
        if isinstance(mesh_term, list):
            all_in = True
            for part in mesh_term:
                if part not in article_mesh_terms:
                    all_in = False
            
            return all_in
        else:
            return mesh_term in article_mesh_terms

def get_date_indices(date, start_date):
    delta_days = (date - start_date).days
    day_index = delta_days

    week_index = (delta_days // 7)

    delta_months = (date.year - start_date.year) * 12 + (date.month - start_date.month)
    month_index = delta_months

    return day_index, week_index, month_index

def add_token_number(article_date, start_date, input_token_num, output_token_num, counts, tokenizer_name, category):
    day_index, week_index, month_index = get_date_indices(article_date, start_date)

    counts[tokenizer_name][category]["day"]["input"][day_index] += input_token_num 
    counts[tokenizer_name][category]["day"]["output"][day_index] += output_token_num 
    counts[tokenizer_name][category]["week"]["input"][week_index] += input_token_num
    counts[tokenizer_name][category]["week"]["output"][week_index] += output_token_num
    counts[tokenizer_name][category]["month"]["input"][month_index] += input_token_num
    counts[tokenizer_name][category]["month"]["output"][month_index] += output_token_num

INTERVALS = [
    "day",
    "week",
    "month"
]

CATEGORIES = [
    "NO KEYWORDS",
    "KEYWORDS",
    "SUBHEADINGS",
    "SITE PROPOSITION",
    "PROPOSITION"
]

TOKENIZERS = [
    #"openai-community/gpt-4",
    #"meta-llama/Llama-2-7b-hf",
    "bert-base-uncased",
    "roberta-base",
    "facebook/bart-large"
]

TMP_DATA_DIR = os.path.abspath(os.path.join(os.path.dirname(__file__), "../../dataSources/PubMed/save"))
RESULT_DATA_DIR = os.path.abspath(os.path.join(os.path.dirname(__file__), "./data"))

ncds_mesh_terms = [mesh_term.lower() for ncd, mesh_term in NCDS_MESH_TERM.items()]
keywords_mesh_terms = lower_keywords(KEYWORDS_MESH_TERM)
keywords_subheading_mesh_terms = lower_keywords(KEYWORDS_MESH_SUBHEADING)
keywords_site_proposition_mesh_terms = lower_keywords(KEYWORDS_MESH_SITE_PROPOSITION)
keywords_proposition_mesh_terms = lower_keywords(KEYWORDS_MESH_PROPOSITION)


file_path = f"{TMP_DATA_DIR}/save_3_years.json"
with open(file_path, "r", encoding="utf-8") as file:
    data = json.load(file)

print(len(data))

counts = {}

for tokenizer_name in TOKENIZERS:
    counts[tokenizer_name] = {}
    counts[tokenizer_name]["ALL"] = {
        "input" : 0,
        "output" : 0
    }
    for category in CATEGORIES:
        counts[tokenizer_name][category] = {
            "day": {
                "input": {},
                "output": {}
            },
            "week": {
                "input": {},
                "output": {}
            },
            "month": {
                "input": {},
                "output": {}
            }
        }

start_date = datetime(2022, 1, 1)
end_date = datetime(2025, 1, 30)
current_date = start_date

while(current_date < end_date):
    day_index, week_index, month_index = get_date_indices(current_date, start_date)

    for tokenizer_name in TOKENIZERS:
        for category in CATEGORIES:
            counts[tokenizer_name][category]["day"]["input"][day_index] = 0
            counts[tokenizer_name][category]["day"]["output"][day_index] = 0
            counts[tokenizer_name][category]["week"]["input"][week_index] = 0
            counts[tokenizer_name][category]["week"]["output"][week_index] = 0
            counts[tokenizer_name][category]["month"]["input"][month_index] = 0
            counts[tokenizer_name][category]["month"]["output"][month_index] = 0

    current_date += timedelta(days=1)



for tokenizer_name in TOKENIZERS:

    tokenizer = AutoTokenizer.from_pretrained(tokenizer_name)

    output_exemple = '```json\n'
    output_exemple += '{'
    output_exemple += '\'labels\': [' + ', '.join(NCDS) + ']'
    output_exemple += '}'
    output_exemple += '```'

    output_tokens = tokenizer(output_exemple, return_tensors="pt")
    output_token_num = len(output_tokens["input_ids"][0])

    i = 1
    for article in data:
        print(f"Article N°{i}")

        article_mesh_terms = [mesh_term.lower() for mesh_term in article["MeshTerms"]]
        article_date = datetime(int(article["Date"]["Year"]), int(article["Date"]["Month"]), int(article["Date"]["Day"]))
        title, abstract = get_article_data(article)

        prompt = 'I need you to give me the labels that could be given to the text (keep in mind that u can put multiple labels and select only the labels that i give you):\n'
        prompt += 'text: ' + title + abstract + '\n'
        prompt += 'labels: ' + "[" + ", ".join(NCDS) + ']\n'
        prompt += 'Give the response in json format: { "labels": [] }. Follow this writing to the letter and don t add text around it. Only put the labels that you select between [].'
        prompt += 'Even if you have no labels give me a response like: { "labels": [] }. And also put labels between "".'

        tokens = tokenizer(prompt, return_tensors="pt")
        num_tokens = len(tokens["input_ids"][0])

        add_token_number(article_date, start_date, num_tokens, output_token_num, counts, tokenizer_name, "NO KEYWORDS")
        counts[tokenizer_name]["ALL"]["input"] += num_tokens
        counts[tokenizer_name]["ALL"]["output"] += output_token_num

        added = False

        for ncd in ncds_mesh_terms:
            if added:
                break

            if mesh_term_present(article_mesh_terms, ncd):

                for keyword in keywords_mesh_terms:
                    if added:
                        break
                    if mesh_term_present(article_mesh_terms, keyword):
                        add_token_number(article_date, start_date, num_tokens, output_token_num, counts, tokenizer_name, "KEYWORDS")
                        add_token_number(article_date, start_date, num_tokens, output_token_num, counts, tokenizer_name, "SUBHEADINGS")
                        add_token_number(article_date, start_date, num_tokens, output_token_num, counts, tokenizer_name, "SITE PROPOSITION")
                        add_token_number(article_date, start_date, num_tokens, output_token_num, counts, tokenizer_name, "PROPOSITION")
                        added = True

                for keyword in keywords_subheading_mesh_terms:
                    if added:
                        break
                    if mesh_term_present(article_mesh_terms, keyword):
                        add_token_number(article_date, start_date, num_tokens, output_token_num, counts, tokenizer_name, "SUBHEADINGS")
                        add_token_number(article_date, start_date, num_tokens, output_token_num, counts, tokenizer_name, "SITE PROPOSITION")
                        add_token_number(article_date, start_date, num_tokens, output_token_num, counts, tokenizer_name, "PROPOSITION")
                        added = True

                for keyword in keywords_site_proposition_mesh_terms:
                    if added:
                        break
                    if mesh_term_present(article_mesh_terms, keyword):
                        add_token_number(article_date, start_date, num_tokens, output_token_num, counts, tokenizer_name, "SITE PROPOSITION")
                        add_token_number(article_date, start_date, num_tokens, output_token_num, counts, tokenizer_name, "PROPOSITION")
                        added = True

                for keyword in keywords_proposition_mesh_terms:
                    if added:
                        break
                    if mesh_term_present(article_mesh_terms, keyword):
                        add_token_number(article_date, start_date, num_tokens, output_token_num, counts, tokenizer_name, "PROPOSITION")
                        added = True

        i += 1

    for category in CATEGORIES:
        for interval in INTERVALS:
            for i in ["input", "output"]:
                counts[tokenizer_name][category][interval][i] = [val for _, val in counts[tokenizer_name][category][interval][i].items()]

                counts[tokenizer_name][category][interval][i] = {
                    "min": min(counts[tokenizer_name][category][interval][i]),
                    "max": max(counts[tokenizer_name][category][interval][i]),
                    "mean": statistics.mean(counts[tokenizer_name][category][interval][i])
                }

with open(f"{RESULT_DATA_DIR}/token_count.json", "w") as json_file:
    json.dump(counts, json_file, indent=4)
