# -----------------------------
# Le script crée les graphes pour chaques catégories de longueur en utilisant les résultats des modèles selectionnés et les stoque dans ./tmp. 
#
# Ce script ne prend pas de paramètres.
#
# Auteur : Pavlovich Ivan
# -----------------------------

import sys
import os
import matplotlib.pyplot as plt
import numpy as np
import json
import itertools
import matplotlib.ticker as ticker
from matplotlib.ticker import MaxNLocator


sys.path.append(os.path.abspath(os.path.join(os.path.dirname(__file__), "../")))

from variables.articles import LENGTH_CATEGORIES
from variables.models import MODELS

def get_filename(model):
    return model.replace(" ", "_").replace("-", "_").replace(".", "_").replace("/", "-")

RESULTS_DIR = os.path.abspath(os.path.join(os.path.dirname(__file__), "./results")) 
TMP_DIR = os.path.abspath(os.path.join(os.path.dirname(__file__), "./tmp"))

length_categories = LENGTH_CATEGORIES + ["ALL"]

models = [
    'fine-tuned/bart-large-mnli',
    'mistral-small',
    # 'llama3.2',
    # 'deepseek-v2',
    # 'gemini-hosted',
    # 'cohere-hosted',
    'facebook/bart-large-mnli',
    # 'MoritzLaurer/DeBERTa-v3-base-mnli-fever-anli',
    'MoritzLaurer/deberta-v3-base-zeroshot-v1.1-all-33',
    # 'MoritzLaurer/multilingual-MiniLMv2-L6-mnli-xnli',
]

metrics = ["TPR", "TNR", "Precision", "Error rate", "F1"]

models_results = {}
usable_models = []

for model in models:
    file_path = f"{RESULTS_DIR}/{get_filename(model)}.json"

    if not os.path.exists(file_path):
        print(f"Warning: File {file_path} not found. Skipping model {model}.")
        continue

    with open(file_path, "r", encoding="utf-8") as file:
        data = json.load(file)

        if "results" not in data:
            print(f"Warning: 'results' key missing in {file_path}. Skipping model {model}.")
            continue

        for length_category in length_categories:
            values = [data["results"][length_category][metric] for metric in metrics]

            data["results"][length_category] = values



        models_results[model] = data["results"]

        usable_models.append(model)

models = usable_models

for length_category in length_categories:
    print(f"GENERATING PLOTS FOR {length_category}")
    width = 0.2
    spacing = width * (len(metrics) + 1)  # Extra spacing between groups
    x = np.arange(len(models)) * spacing 

    fig, ax = plt.subplots(figsize=(14, 8))

    for i in range(len(metrics)):
        values = np.array([models_results[model][length_category][i] for model in models])
        bars = ax.bar(x + i * width, values, width, label=metrics[i])

        # Add values on top of bars
        for bar in bars:
            height = bar.get_height()
            ax.text(
                bar.get_x() + bar.get_width() / 2, 
                height, 
                f"{height:.2f}",  # Format to 2 decimal places
                ha='center', 
                va='bottom', 
                fontsize=10
            )

    ax.set_xticks(x + width * (len(metrics) / 2 - 0.5))
    ax.set_xticklabels(models, rotation=20, ha="right")

    ax.yaxis.set_major_locator(MaxNLocator(nbins=10))
    ax.yaxis.set_minor_locator(ticker.AutoMinorLocator(2))

    ax.set_ylabel("Score")
    name = "Comparaison des grands modèles linguistiques pour"

    if length_category == "ALL":
        name += " toutes les données"
    elif length_category == "SHORT":
        name += " les textes courts"
    elif length_category == "MEDIUM":
        name += " les textes moyens"
    elif length_category == "LONG":
        name += " les textes longs"
    elif length_category == "VERY LONG":
        name += " les textes très longs"

    name = ""
    
    ax.set_title(name)
    ax.legend()
    
    plt.tight_layout()
    plt.savefig(f"{TMP_DIR}/{get_filename(length_category)}_Fine-Tuned.png")
    plt.close(fig)
