# -----------------------------
# Le script crée des bar chart et spider chart pour toutes les combinaisons de modèles possible et sotque dans ./results/graphs.
#
# Ce script ne prend pas de paramètres.
#
# Auteur : Pavlovich Ivan
# -----------------------------

import sys
import os
import matplotlib.pyplot as plt
import numpy as np
import json
import itertools
import matplotlib.ticker as ticker
from matplotlib.ticker import MaxNLocator


sys.path.append(os.path.abspath(os.path.join(os.path.dirname(__file__), "../")))

from variables.articles import LENGTH_CATEGORIES
from variables.models import MODELS

def get_filename(model):
    return model.replace(" ", "_").replace("-", "_").replace(".", "_").replace("/", "-")

RESULTS_DIR = os.path.abspath(os.path.join(os.path.dirname(__file__), "./results")) 
GRAPHS_DIR = os.path.abspath(os.path.join(os.path.dirname(__file__), "./results/graphs"))

length_categories = LENGTH_CATEGORIES + ["ALL"]

models = [model for model in MODELS.keys()]

metrics = ["TPR", "TNR", "Precision", "Error rate", "F1"]

models_results = {}
usable_models = []

for model in models:
    file_path = f"{RESULTS_DIR}/{get_filename(model)}.json"

    if not os.path.exists(file_path):
        print(f"Warning: File {file_path} not found. Skipping model {model}.")
        continue

    with open(file_path, "r", encoding="utf-8") as file:
        data = json.load(file)

        if "results" not in data:
            print(f"Warning: 'results' key missing in {file_path}. Skipping model {model}.")
            continue

        for length_category in length_categories:
            values = [data["results"][length_category][metric] for metric in metrics]

            data["results"][length_category] = values

        models_results[model] = data["results"]
        usable_models.append(model)

models = usable_models

print("GENERATING BAR PLOTS")

os.makedirs(f"{GRAPHS_DIR}/BAR", exist_ok=True)
for length_category in length_categories:
    os.makedirs(f"{GRAPHS_DIR}/BAR/{get_filename(length_category)}", exist_ok=True)
    for n in range(2, len(models)):
        os.makedirs(f"{GRAPHS_DIR}/BAR/{get_filename(length_category)}/{n}", exist_ok=True) 

for length_category in length_categories:
    print(f"GENERATING PLOTS FOR {length_category}")
    for n in range(2, len(models)):
        for selected_models in itertools.combinations(models, n):

            x = np.arange(len(selected_models))
            width = 0.2

            fig, ax = plt.subplots(figsize=(14, 8))

            for i in range(len(metrics)):
                ax.bar(x + i * width, np.array([models_results[model][length_category][i] for model in selected_models]), width, label=metrics[i])

            ax.set_xticks(x + width * (len(metrics) / 2 - 0.5))
            ax.set_xticklabels(selected_models, rotation=20, ha="right")

            ax.yaxis.set_major_locator(MaxNLocator(nbins=10))
            ax.yaxis.set_minor_locator(ticker.AutoMinorLocator(2))

            ax.set_ylabel("Score")
            ax.set_title("Comparaison des modèles selon les métriques")
            ax.legend()

            filename = "&".join([get_filename(model) for model in selected_models])
            
            plt.tight_layout()
            plt.savefig(f"{GRAPHS_DIR}/BAR/{get_filename(length_category)}/{len(selected_models)}/{filename}.png")
            plt.close(fig)

print("GENERATING SPIDER CHARTS")

os.makedirs(f"{GRAPHS_DIR}/SPIDER", exist_ok=True)
for length_category in length_categories:
    os.makedirs(f"{GRAPHS_DIR}/SPIDER/{get_filename(length_category)}", exist_ok=True) 
    for n in range(2, 4):
        os.makedirs(f"{GRAPHS_DIR}/SPIDER/{get_filename(length_category)}/{n}", exist_ok=True) 

for length_category in length_categories:
    print(f"GENERATING CHARTS FOR {length_category}")
    for n in range(2, 4):
        for selected_models in itertools.combinations(models, n):

            num_vars = len(metrics)
            angles = np.linspace(0, 2 * np.pi, num_vars, endpoint=False).tolist()
            angles += angles[:1]

            # Create Radar Chart
            fig, ax = plt.subplots(figsize=(14, 6), subplot_kw=dict(polar=True))

            # Plot each model
            for i, model in enumerate(selected_models):
                ax.plot(angles, models_results[model][length_category] + [models_results[model][length_category][0]], label=model, linewidth=2, linestyle='solid')
                ax.fill(angles, models_results[model][length_category] + [models_results[model][length_category][0]], alpha=0.1)

            # Labels
            ax.set_xticks(angles[:-1])
            ax.set_xticklabels(metrics)
            plt.title("Comparison of Models on Different Metrics", pad=30, fontsize=12)
            fig.subplots_adjust(top=0.9)
            plt.legend(loc="upper right", bbox_to_anchor=(1.8, 1))

            filename = "&".join([get_filename(model) for model in selected_models])

            plt.savefig(f"{GRAPHS_DIR}/SPIDER/{get_filename(length_category)}/{n}/{filename}.png")
            plt.close(fig)
