# -----------------------------
# Le script lance la classification du jeu de données de test avec un modèle selectionné.
#
# Arguments:
# --model : nom du modèle présent dans le fichier variables/models.py
# --remake : permet de refaire les tests déjà réaliser (supprime les anciens résultats)
#
# Auteur : Pavlovich Ivan
# -----------------------------

import sys
import os
import statistics
import time
import json
import argparse

# Ajouter le répertoire parent au chemin de recherche
sys.path.append(os.path.abspath(os.path.join(os.path.dirname(__file__), "../")))

from variables.pubmed import NCDS
from variables.huggingface import HUGGINGFACE_MODELS
from variables.articles import LENGTH_CATEGORIES, LENGTH_CATEGORIES_TRESHOLDS
from variables.models import MODELS
from testModel.utils import get_dataset_filename, get_article_data, get_wanted_predictions, save_model_json, save_model_results_json, create_model_data_var, store_results
from testModel.metrics import get_confusion_matrix
from parsers.jsonParser import parseJsonFile

RESULTS_DIR_NAME = "./results"
DATASET_DIR_NAME = "./dataset"

RESULTS_DIR = os.path.abspath(os.path.join(os.path.dirname(__file__), RESULTS_DIR_NAME))
DATASET_DIR = os.path.abspath(os.path.join(os.path.dirname(__file__), DATASET_DIR_NAME))

TRESHOLD = 0.7

parser = argparse.ArgumentParser(description="Script for article classification.")
parser.add_argument('--model', '-m', type=str, required=True, help='The model to use for classification (mandatory).')
parser.add_argument('--remake', '-r', action='store_true', help='Flag to remake all the tests (optional).')

args = parser.parse_args()

model = args.model
remake = args.remake

if model not in MODELS.keys():
    print(f"Model {model} isn't supported!")
    print(f"Models: {MODELS.keys()}")
    exit(1)

result_folder_path = f"{RESULTS_DIR}"

result_filename = model.replace(" ", "_").replace("-", "_").replace(".", "_").replace("/", "-")

file_path = f"{result_folder_path}/{result_filename}.json"

if os.path.exists(file_path) and remake:
    os.remove(file_path)
    print(f"File {file_path} has been deleted.")
else:
    print(f"File {file_path} does not exist.")

if not os.path.exists(file_path):
    initial_data = create_model_data_var(NCDS)
    print(initial_data)
    with open(file_path, "w", encoding="utf-8") as file:
        json.dump(initial_data, file, indent=4)

with open(file_path, "r", encoding="utf-8") as file:
    data = json.load(file)

if data["treshold"] is None:
    data["treshold"] = TRESHOLD

pmids = [article["PMID"] for article in data["articles"]]

if MODELS[model]["requiresDependency"]:
    if MODELS[model]["category"] == "huggingface":
        dependency = MODELS[model]["dependency"](model)
    elif MODELS[model]["category"] == "hosted":
        dependency = MODELS[model]["dependency"]()
    elif MODELS[model]["category"] == "fine-tuned":
        dependency = MODELS[model]["dependency"]()

print("---------------------------------")
print(f"MODEL: {model}")
print(f"TRESHOLD: {data['treshold']}")
print("---------------------------------")

for ncd in NCDS:

    try:
        filename = get_dataset_filename(ncd)
        articles = parseJsonFile(f"{DATASET_DIR}/{filename}.json")
    except Exception as e:
        print(f"Error: {e}")

    for article in articles:

        if article['PMID'] in pmids:
            print("---------------------------------")
            print(f"Article {article['PMID']} already classified!")
            print("---------------------------------")
            continue

        article_result = {}

        print("---------------------------------")

        title, abstract = get_article_data(article)
        wanted = get_wanted_predictions(article, NCDS)

        start = time.time()

        if MODELS[model]["category"] == "huggingface":
            predictions, predictionTime = MODELS[model]["predict"](dependency, title+abstract, NCDS, data["treshold"])
        elif MODELS[model]["category"] == "ollama":
            predictions, predictionTime = MODELS[model]["predict"](model, title+abstract, NCDS)
        elif MODELS[model]["category"] == "hosted":
            predictions, predictionTime = MODELS[model]["predict"](dependency, title+abstract, NCDS)
        elif MODELS[model]["category"] == "fine-tuned":
            predictions, predictionTime = MODELS[model]["predict"](dependency, title+abstract, NCDS, data["treshold"])

        end = time.time()

        selected_labels = [key for key, value in predictions.items() if value]

        article_result["PMID"] = article['PMID']
        article_result["Labels"] = article["Predictions"]
        article_result["MeshTerms"] = article["MeshTerms"]
        article_result["Wanted"] = wanted
        article_result["Predictions"] = predictions
        article_result["Selected Labels"] = selected_labels
        article_result["confusion matrix"] = get_confusion_matrix(wanted, predictions)
        data["articles"].append(article_result)

        print(f"PMID: {article_result['PMID']}")
        print(f"Labels: {article_result['Labels']}")
        print(f"MeshTerm: {article_result['MeshTerms']}")
        print(f"Wanted: {article_result['Wanted']}")
        print(f"Predicted: {article_result['Predictions']}")
        print(f"Selected labels: {article_result['Selected Labels']}")
        print(f"Confusion matrix: {article_result['confusion matrix']}")
        
        data["results"]["ALL"] = store_results(data["results"]["ALL"], wanted, predictions, predictionTime, end - start)

        added = False
        for id, length_category_treshold in enumerate(LENGTH_CATEGORIES_TRESHOLDS):
            if len(title+abstract) < length_category_treshold:
                data["results"][LENGTH_CATEGORIES[id]] = store_results(data["results"][LENGTH_CATEGORIES[id]], wanted, predictions, predictionTime, end - start)
                added = True
                break

        if not added:
            data["results"][LENGTH_CATEGORIES[-1]] = store_results(data["results"][LENGTH_CATEGORIES[-1]], wanted, predictions, predictionTime, end - start)

        print("---------------------------------")

        with open(file_path, "w", encoding="utf-8") as file:
            json.dump(data, file, indent=4)

for length_category in LENGTH_CATEGORIES:

    print(f"Name: {length_category}")
    print(f"Time to classify articles: {data['results'][length_category]['prediction time']} seconds")
    print(f"Time to gather predictions: {data['results'][length_category]['gathering results time']} seconds")
    print(f"Result confusion matrix: {data['results'][length_category]['confusion matrix']}")
    print(f"True Positive Rate (TPR): {data['results'][length_category]['TPR']}")
    print(f"True Negative Rate (TNR): {data['results'][length_category]['TNR']}")
    print(f"Precision: {data['results'][length_category]['Precision']}")
    print(f"Error Rate: {data['results'][length_category]['Error rate']}")
    print(f"F1 Score: {data['results'][length_category]['F1']}")
    print("---------------------------------")

print(f"Name: ALL")
print(f"Time to classify all articles: {data['results']['ALL']['prediction time']} seconds")
print(f"Time to gather all predictions: {data['results']['ALL']['gathering results time']} seconds")
print(f"Result confusion matrix: {data['results']['ALL']['confusion matrix']}")
print(f"True Positive Rate (TPR): {data['results']['ALL']['TPR']}")
print(f"True Negative Rate (TNR): {data['results']['ALL']['TNR']}")
print(f"Precision: {data['results']['ALL']['Precision']}")
print(f"Error Rate: {data['results']['ALL']['Error rate']}")
print(f"F1 Score: {data['results']['ALL']['F1']}")
print()